/*
 * QTI DIRECTEMP(R) USB driver
 *
 * Copyright (c) 2010 Cyber Switching, Inc.
 *    Chris Verges <chrisv@cyberswitching.com>
 *
 * Copyright (c) 2010 Quality Thermistor, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, version 2.
 */

#include <linux/kernel.h>
#include <linux/ctype.h>
#include <linux/errno.h>
#include <linux/hwmon.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/usb.h>

#define DRV_NAME		"directemp"
#define DRV_VERSION		"0.5"
#define DRV_AUTHOR		"Chris Verges <chrisv@cyberswitching.com>"
#define DRV_DESCRIPTION		"QTI DIRECTEMP(R) USB driver"
#define DRV_RELDATE		"Apr 9, 2010"

#define DIRECTEMP_DEBUG 0

#if DIRECTEMP_DEBUG
	#ifdef dev_dbg
		#undef dev_dbg
	#endif
	#define dev_dbg(...)	dev_info(__VA_ARGS__)
#endif

#define QTI_VENDOR_ID		(0x1DFD)

#define NUM_SENSOR_TYPES	(10)
#define SENSOR_ACIDITY		'A'
#define SENSOR_DISPLACEMENT	'D'
#define SENSOR_FLOW		'F'
#define SENSOR_HUMIDITY		'H'
#define SENSOR_LIGHT		'L'
#define SENSOR_MOTION		'M'
#define SENSOR_PRESSURE		'P'
#define SENSOR_SWITCH_CONTACT	'S'
#define SENSOR_TEMPERATURE	'T'
#define SENSOR_VOLTAGE		'V'

#define DIRECTEMP_ERROR		'E'

#define DIRECTEMP_MSG_SIZE	(16)
#define DIRECTEMP_TIMEOUT	(1000)
#define DIRECTEMP_MAX_RETRIES	(3)
#define DIRECTEMP_MAX_SENSORS	(10)
#define DIRECTEMP_SYSFS_LEN	(16)

struct directemp_sensor {
	char			*name;
	char			id;
	char			type;
	struct device_attribute	attr;

	ssize_t (*read)(struct device *dev, struct device_attribute *attr,
			char *buf);
	ssize_t (*write)(struct device *dev, struct device_attribute *attr,
			const char *buf, size_t count);
};

struct usb_directemp {
	struct usb_device	*udev;
	struct usb_interface	*interface;
	uint16_t		usb_pid;

	struct directemp_sensor	*sensors;
	int			num_sensors;

	struct device		*hwmon_dev;
	struct mutex		lock;		/* single access lock */
};

struct directemp_error {
	char			*code;
	char			*msg;
};

static struct directemp_error errors[] = {
	{"301", "sensor short circuit"},
	{"302", "sensor open circuit"},
	{"319", "operation not permitted or no sensor at channel specified"},
	{"333", "command not recognized"},
	{NULL,  NULL}
};

static void directemp_hex_dump(const char *caption, void *data, int len)
{
#if DIRECTEMP_DEBUG
	char *tmp = data;
	int i;

	printk("%s", caption);
	for (i = 0; i < len; i++)
		printk("0x%02X ", tmp[i]);
	printk("\n");

	for (i = strlen(caption); i > 0; i--)
		printk(" ");
	for (i = 0; i < len; i++) {
		if (isalnum(tmp[i]) || ispunct(tmp[i]))
			printk("%-4c ", tmp[i]);
		else if (tmp[i] == '\0')
			printk("---- ");
		else if (tmp[i] == '\r')
			printk("%-4s ", "\\r");
		else if (tmp[i] == '\n')
			printk("%-4s ", "\\n");
		else if (tmp[i] == '\t')
			printk("%-4s ", "\\t");
		else
			printk("???? ");
	}
	printk("\n");
#endif
}

static int directemp_read(struct usb_device *dev, void *data, int len)
{
	int actual_len;
	int err;

	err = usb_interrupt_msg(dev, usb_rcvintpipe(dev, 1),
			data, len, &actual_len,	DIRECTEMP_TIMEOUT);
	if (err)
		return err;

	directemp_hex_dump("Reply: ", data, len);

	return actual_len;
}

static int directemp_write(struct usb_device *dev, void *data, int len)
{
	int actual_len;
	int err;

	directemp_hex_dump("Query: ", data, len);

	err = usb_interrupt_msg(dev, usb_sndintpipe(dev, 2),
			data, len, &actual_len,	DIRECTEMP_TIMEOUT);
	if (err)
		return err;

	return actual_len;
}

static struct directemp_sensor *get_sensor(struct usb_directemp *dev,
				const char *name)
{
	int i;
	for (i = 0; i < dev->num_sensors; i++)
		if (name == dev->sensors[i].name)
			return &dev->sensors[i];
	return NULL;
}

static int iswhitespace(char c)
{
	return isspace(c) || c == '\r' || c == '\n' || c == '\0';
}

static ssize_t directemp_rtrim(char *buf, ssize_t len)
{
	while (len > 0 && iswhitespace(buf[len - 1]))
		len--;
	buf[len] = '\0';

	return len;
}

static char *directemp_errmsg(const char *code)
{
	struct directemp_error *err = errors;
	int len;

	while (err->code) {
		len = strlen(err->code);
		if (!strncasecmp(code, err->code, len))
			return err->msg;
		err++;
	}

	return "(unknown error code)";
}

static int directemp_query(struct usb_directemp *directemp,
				char *tx, size_t tx_len,
				char *rx, size_t rx_max_len, size_t *rx_len)
{
	struct device *dev = &directemp->udev->dev;
	int err;

	err = directemp_write(directemp->udev, tx, tx_len);
	if (err < 0) {
		dev_err(dev, "error %d during communication\n", err);
		return -EAGAIN;
	}
	dev_dbg(dev, "transferred %d bytes\n", err);

	memset(rx, 0, rx_max_len * sizeof(char));
	err = directemp_read(directemp->udev, rx, rx_max_len);
	if (err < 0) {
		dev_err(dev, "error %d during communication\n", err);
		return -EAGAIN;
	}
	dev_dbg(dev, "transferred %d bytes\n", err);

	if (rx_len)
		*rx_len = directemp_rtrim(rx, err);

	return 0;
}

/*
 * Query format:
 *    2
 * Response format:
 *    2vvvvvvvvvvvvvvv
 *
 * where
 *    v    is the reading from the sensor
 *
 * The response code MAY be terminated by either a NULL character (\0)
 * or CRLF (\r\n).
 */
static ssize_t directemp_query_v1(struct device *dev,
				struct device_attribute *attr, char *buf)
{
	struct usb_interface *intf = to_usb_interface(dev);
	struct usb_directemp *directemp = usb_get_intfdata(intf);
	struct directemp_sensor *sensor;
	char tx[DIRECTEMP_MSG_SIZE];
	char rx[DIRECTEMP_MSG_SIZE];
	int retry;
	int err;

	sensor = get_sensor(directemp, attr->attr.name);
	if (!sensor) {
		dev_err(dev, "unknown sensor");
		return 0;
	}

	memset(tx, 0, sizeof(tx));
	tx[0] = '2';

	mutex_lock(&directemp->lock);

	for (retry = 0; retry < DIRECTEMP_MAX_RETRIES; retry++) {
		err = directemp_query(directemp, tx, DIRECTEMP_MSG_SIZE,
				rx, DIRECTEMP_MSG_SIZE, NULL);
		if (err)
			continue;

		if (rx[0] == tx[0]) {
			directemp_rtrim(rx, err);
			if (strlen(&rx[1]) > 0) {
				mutex_unlock(&directemp->lock);
				return sprintf(buf, "%s\n", &rx[1]);
			}
			dev_dbg(dev, "no value detected in response\n");
		} else {
			dev_dbg(dev, "reply does not match query\n");
		}
	}

	if (retry >= DIRECTEMP_MAX_RETRIES)
		dev_err(dev, "exceeded maximum retry count\n");

	mutex_unlock(&directemp->lock);
	return -EAGAIN;
}

/*
 * Query format:
 *    Qn
 * Response format:
 *    Eccc
 *    QnEccc
 *    Qntvvvvvvvvvvvvv
 *
 * where
 *    n    is the sensor number (0 to 9)
 *    c    is the error code
 *    t    is the sensor type code
 *    v    is the reading from the sensor
 *
 * The response code MAY be terminated by either a NULL character (\0)
 * or CRLF (\r\n).
 */
static ssize_t directemp_query_v2(struct device *dev,
				struct device_attribute *attr, char *buf)
{
	struct usb_interface *intf = to_usb_interface(dev);
	struct usb_directemp *directemp = usb_get_intfdata(intf);
	struct directemp_sensor *sensor;
	char rx[DIRECTEMP_MSG_SIZE];
	char tx[DIRECTEMP_MSG_SIZE];
	size_t rx_len = 0;
	int retry;
	int err;

	sensor = get_sensor(directemp, attr->attr.name);
	if (!sensor) {
		dev_err(dev, "unknown sensor\n");
		return 0;
	}

	memset(tx, 0, sizeof(tx));
	tx[0] = 'Q';
	tx[1] = sensor->id;

	mutex_lock(&directemp->lock);
	
	for (retry = 0; retry < DIRECTEMP_MAX_RETRIES; retry++) {
		err = directemp_query(directemp, tx, DIRECTEMP_MSG_SIZE,
				rx, DIRECTEMP_MSG_SIZE, &rx_len);
		if (err)
			continue;

		if (rx_len < 1) {
			dev_dbg(dev, "no response from sensor\n");
			continue;
		}

		if (rx[0] == DIRECTEMP_ERROR) {
			dev_err(dev, "error %s: %s\n", &rx[1],
					directemp_errmsg(&rx[1]));
			continue;
		}

		if (rx[0] != tx[0]) {
			dev_err(dev, "reply does not match query\n");
			continue;
		}

		if (rx[1] != tx[1]) {
			dev_dbg(dev, "reply not for requested sensor number\n");
			continue;
		}

		if (rx[2] == DIRECTEMP_ERROR) {
			dev_err(dev, "error %s: %s\n", &rx[3],
					directemp_errmsg(&rx[3]));
			continue;
		}

		if (rx[2] != sensor->type) {
			dev_dbg(dev, "reply does not match expected type\n");
			continue;
		}

		if (rx_len <= 3) {
			dev_dbg(dev, "no value detected in response\n");
			continue;
		}

		mutex_unlock(&directemp->lock);
		return sprintf(buf, "%s\n", &rx[3]);
	}

	if (retry >= DIRECTEMP_MAX_RETRIES)
		dev_err(dev, "exceeded maximum retry count\n");

	mutex_unlock(&directemp->lock);
	return -EAGAIN;
}

static ssize_t directemp_query_v2_boardinfo(struct device *dev,
				struct device_attribute *attr, char *buf,
				char query)
{
	struct usb_interface *intf = to_usb_interface(dev);
	struct usb_directemp *directemp = usb_get_intfdata(intf);
	char rx[DIRECTEMP_MSG_SIZE];
	char tx[DIRECTEMP_MSG_SIZE];
	size_t rx_len = 0;
	int retry;
	int err;

	memset(tx, 0, sizeof(tx));
	tx[0] = query;

	mutex_lock(&directemp->lock);

	for (retry = 0; retry < DIRECTEMP_MAX_RETRIES; retry++) {
		err = directemp_query(directemp, tx, DIRECTEMP_MSG_SIZE,
				rx, DIRECTEMP_MSG_SIZE, &rx_len);
		if (err)
			continue;

		if (rx_len < 1) {
			dev_dbg(dev, "no response\n");
			continue;
		}

		if (rx[0] == DIRECTEMP_ERROR) {
			dev_err(dev, "error %s: %s\n", &rx[1],
					directemp_errmsg(&rx[1]));
			continue;
		}

		if (rx[0] != tx[0]) {
			dev_err(dev, "reply does not match query\n");
			continue;
		}

		if (rx_len <= 1) {
			dev_dbg(dev, "no value detected in response\n");
			continue;
		}

		mutex_unlock(&directemp->lock);
		return sprintf(buf, "%s\n", &rx[1]);
	}

	if (retry >= DIRECTEMP_MAX_RETRIES)
		dev_err(dev, "exceeded maximum retry count\n");

	mutex_unlock(&directemp->lock);
	return -EAGAIN;
}

static ssize_t directemp_query_v2_hw_version(struct device *dev,
				struct device_attribute *attr, char *buf)
{
	return directemp_query_v2_boardinfo(dev, attr, buf, 'B');
}

static ssize_t directemp_query_v2_sw_version(struct device *dev,
				struct device_attribute *attr, char *buf)
{
	return directemp_query_v2_boardinfo(dev, attr, buf, 'V');
}

static ssize_t directemp_query_v2_serial_number(struct device *dev,
				struct device_attribute *attr, char *buf)
{
	return directemp_query_v2_boardinfo(dev, attr, buf, 'S');
}

static DEVICE_ATTR(hw_version, S_IRUGO, directemp_query_v2_hw_version, NULL);
static DEVICE_ATTR(sw_version, S_IRUGO, directemp_query_v2_sw_version, NULL);
static DEVICE_ATTR(serial_number, S_IRUGO, directemp_query_v2_serial_number, NULL);

static ssize_t show_raw(struct device *dev, struct device_attribute *attr,
				char *buf)
{
	struct usb_interface *intf = to_usb_interface(dev);
	struct usb_directemp *directemp = usb_get_intfdata(intf);
	struct usb_device *udev = directemp->udev;
	char data[DIRECTEMP_MSG_SIZE];
	int len = 0;
	int i;

	memset(data, 0, sizeof(data));

	mutex_lock(&directemp->lock);
	directemp_read(udev, data, DIRECTEMP_MSG_SIZE);
	mutex_unlock(&directemp->lock);

	for (i = 0; i < DIRECTEMP_MSG_SIZE; i++) {
		unsigned int tmp = data[i] & 0x000000FF;
		len += sprintf(&buf[len], "0x%02X ", tmp);
	}
	len += sprintf(&buf[len], "\n");

	for (i = 0; i < DIRECTEMP_MSG_SIZE; i++) {
		if (isalnum(data[i]) || ispunct(data[i]))
			len += sprintf(&buf[len], "%-4c ", data[i]);
		else if (data[i] == '\0')
			len += sprintf(&buf[len], "---- ");
		else if (data[i] == '\r')
			len += sprintf(&buf[len], "%-4s ", "\\r");
		else if (data[i] == '\n')
			len += sprintf(&buf[len], "%-4s ", "\\n");
		else if (data[i] == '\t')
			len += sprintf(&buf[len], "%-4s ", "\\t");
		else
			len += sprintf(&buf[len], "???? ");
	}
	len += sprintf(&buf[len], "\n");

	return len;
}

static ssize_t show_raw_simple(struct device *dev, struct device_attribute *attr,
				char *buf)
{
	struct usb_interface *intf = to_usb_interface(dev);
	struct usb_directemp *directemp = usb_get_intfdata(intf);
	struct usb_device *udev = directemp->udev;
	char data[DIRECTEMP_MSG_SIZE];

	memset(data, 0, sizeof(data));

	mutex_lock(&directemp->lock);
	directemp_read(udev, data, DIRECTEMP_MSG_SIZE);
	mutex_unlock(&directemp->lock);

	/*
	 * Just blindly dump the received output, unprintable chars
	 * and all.
	 */
	return sprintf(buf, "%s", data);
}

static ssize_t store_raw(struct device *dev, struct device_attribute *attr,
				const char *buf, size_t count)
{
	struct usb_interface *intf = to_usb_interface(dev);
	struct usb_directemp *directemp = usb_get_intfdata(intf);
	struct usb_device *udev = directemp->udev;
	char data[DIRECTEMP_MSG_SIZE];
	int i;

	memset(data, 0, sizeof(data));
	for (i = 0; i < DIRECTEMP_MSG_SIZE && i < count; i++)
		data[i] = buf[i];

	mutex_lock(&directemp->lock);
	directemp_write(udev, data, DIRECTEMP_MSG_SIZE);
	mutex_unlock(&directemp->lock);

	return min_t(size_t, count, DIRECTEMP_MSG_SIZE);
}

static DEVICE_ATTR(raw, S_IRUSR | S_IWUSR, show_raw, store_raw);
static DEVICE_ATTR(raw_simple, S_IRUSR | S_IWUSR, show_raw_simple, store_raw);

static __devinit int init_directemp_0002(struct usb_interface *interface,
				struct usb_directemp *dev)
{
	struct directemp_sensor *sensor;
	int err;

	dev->sensors = kzalloc(1 * sizeof(struct directemp_sensor), GFP_KERNEL);
	if (!dev->sensors) {
		dev->num_sensors = 0;
		return -ENOMEM;
	}
	dev->num_sensors = 1;

	sensor = &dev->sensors[0];
	sensor->name = kmalloc(13 * sizeof(char), GFP_KERNEL);
	if (!sensor->name) {
		dev_err(&dev->udev->dev, "out of memory while "
				"naming sensor 0\n");
		return -ENOMEM;
	}
	sprintf(sensor->name, "temp1_input");
	dev_dbg(&dev->udev->dev, "sensor 0: %s\n", sensor->name);

	sensor->id = '0';
	sensor->type = '2';
	sensor->attr.attr.name = sensor->name;
	sensor->attr.attr.mode = S_IRUGO;
	sensor->attr.show = directemp_query_v1;
	sensor->attr.store = NULL;
	err = device_create_file(&interface->dev, &sensor->attr);
	if (err)
		return err;

	return 0;
}

/*
 * Each sensor channel on the newer DIRECTEMP(R) devices will report
 * their data type in the response packet.  During initialization, scan
 * the sensors for this data type and use it to register the hwmon
 * sysfs hooks.  In this way, we can dynamically support a large number
 * of DIRECTEMP(R) models without the need for a lot of manual labor.
 */
static __devinit int init_directemp_sensors(struct usb_interface *interface,
				struct usb_directemp *dev)
{
	struct directemp_sensor *sensor;
	char sensors[DIRECTEMP_MAX_SENSORS];
	char rx[DIRECTEMP_MSG_SIZE];
	char tx[DIRECTEMP_MSG_SIZE];
	const char *format;
	uint8_t counts[NUM_SENSOR_TYPES];
	uint8_t *count;
	size_t rx_len = 0;
	int num_sensors = 0;
	int retry;
	int err;
	int i;
	int j;

	/*
	 * Handle the legacy 0x0002 devices separately since they
	 * are not self-reporting.
	 */
	if (dev->usb_pid == 0x0002)
		return init_directemp_0002(interface, dev);

	/* For the rest, query each channel to find the sensor type */
	memset(sensors, 0, sizeof(sensors));
	memset(counts, 0, sizeof(counts));
	memset(tx, 0, sizeof(tx));
	tx[0] = 'Q';

	for (i = 0; i < DIRECTEMP_MAX_SENSORS; i++) {
		dev_dbg(&dev->udev->dev, "querying sensor %d\n", i);
		tx[1] = '0' + i;	/* Convert i to a char */

		mutex_lock(&dev->lock);

		for (retry = 0; retry < DIRECTEMP_MAX_RETRIES; retry++) {
			err = directemp_query(dev, tx, DIRECTEMP_MSG_SIZE,
					rx, DIRECTEMP_MSG_SIZE, &rx_len);
			if (err)
				continue;

			if (rx_len < 1) {
				dev_dbg(&dev->udev->dev,
						"no response from sensor\n");
				continue;
			}

			if (rx[0] == DIRECTEMP_ERROR) {
				dev_dbg(&dev->udev->dev, "error %s: %s\n",
						&rx[1],
						directemp_errmsg(&rx[1]));
				/*
				 * This response may be a buffer offset
				 * problem due to the last query.  We need
				 * to retry, unfortunately.
				 */
				continue;
			}

			if (rx[0] != tx[0]) {
				dev_err(&dev->udev->dev, "reply does not "
						"match query\n");
				continue;
			}

			if (rx[1] != tx[1]) {
				dev_dbg(&dev->udev->dev, "reply not for "
						"requested sensor number\n");
				continue;
			}

			if (rx[2] == DIRECTEMP_ERROR) {
				dev_dbg(&dev->udev->dev, "error %s: %s\n",
						&rx[3],
						directemp_errmsg(&rx[3]));
				/*
				 * Since the command and channel number
				 * match, we can assume that this error
				 * response is valid and not a buffer
				 * offset problem.  Stop processing
				 * this channel and move to the next one.
				 */
				break;
			}

			if (rx_len <= 3) {
				dev_dbg(&dev->udev->dev, "no value detected "
						"in response\n");
				continue;
			}

			sensors[i] = rx[2];
			num_sensors++;
			break;
		}

		mutex_unlock(&dev->lock);

		if (retry >= DIRECTEMP_MAX_RETRIES)
			dev_dbg(&dev->udev->dev, "exceeded maximum retry "
					"count\n");
	}

	if (!num_sensors) {
		dev_err(&dev->udev->dev, "no sensors found!\n");
		return -ENODEV;
	}

	/* Allocate the proper number of sensors */
	dev->sensors = kzalloc(num_sensors * sizeof(struct directemp_sensor),
			GFP_KERNEL);
	if (!dev->sensors) {
		dev->num_sensors = 0;
		return -ENOMEM;
	}
	dev->num_sensors = num_sensors;

	/* Setup each sensor in hwmon/sysfs */
	for (i = 0, j = 0; i < DIRECTEMP_MAX_SENSORS; i++) {
		/* Skip this sensor channel */
		if (sensors[i] == '\0') {
			dev_dbg(&dev->udev->dev, "sensor %d: unused\n", i);
			continue;
		}

		switch (sensors[i]) {
		case SENSOR_ACIDITY:
			format = "ph%d_input";
			count = &counts[0];
			break;
		case SENSOR_DISPLACEMENT:
			format = "displacement%d_input";
			count = &counts[1];
			break;
		case SENSOR_FLOW:
			format = "flow%d_input";
			count = &counts[2];
			break;
		case SENSOR_HUMIDITY:
			format = "humidity%d_input";
			count = &counts[3];
			break;
		case SENSOR_LIGHT:
			format = "photo%d_input";
			count = &counts[4];
			break;
		case SENSOR_MOTION:
			format = "motion%d_input";
			count = &counts[5];
			break;
		case SENSOR_PRESSURE:
			format = "pressure%d_input";
			count = &counts[6];
			break;
		case SENSOR_SWITCH_CONTACT:
			format = "switch%d_input";
			count = &counts[7];
			break;
		case SENSOR_TEMPERATURE:
			format = "temp%d_input";
			count = &counts[8];
			break;
		case SENSOR_VOLTAGE:
			format = "in%d_input";
			count = &counts[9];
			break;
		default:
			format = NULL;
			count = NULL;
			break;
		}

		if (!format) {
			dev_dbg(&dev->udev->dev, "sensor %d: unknown type "
					"'%c'\n", i, sensors[i]);
			continue;
		}

		(*count)++;

		sensor = &dev->sensors[j++];

		sensor->name = kmalloc(strlen(format) * sizeof(char),
				GFP_KERNEL);
		if (!sensor->name) {
			dev_err(&dev->udev->dev, "out of memory while "
					"naming sensor %d\n", i);
			return -ENOMEM;
		}
		sprintf(sensor->name, format, *count);
		dev_info(&dev->udev->dev, "sensor %d: %s\n", i, sensor->name);

		sensor->id = '0' + i;	/* Convert i to a char */
		sensor->type = sensors[i];
		sensor->attr.attr.name = sensor->name;
		sensor->attr.attr.mode = S_IRUGO;
		sensor->attr.show = directemp_query_v2;
		sensor->attr.store = NULL;
		err = device_create_file(&interface->dev, &sensor->attr);
		if (err)
			return err;
	}

	err = device_create_file(&interface->dev, &dev_attr_hw_version);
	if (err)
		return err;

	err = device_create_file(&interface->dev, &dev_attr_sw_version);
	if (err)
		return err;

	err = device_create_file(&interface->dev, &dev_attr_serial_number);
	if (err)
		return err;

	return 0;
}

static void free_directemp_sensors(struct usb_interface *interface,
				struct usb_directemp *dev)
{
	if (!dev->num_sensors)
		return;

	while (dev->num_sensors > 0) {
		struct directemp_sensor *sensor;
		sensor = &dev->sensors[--dev->num_sensors];

		device_remove_file(&interface->dev, &sensor->attr);
		kfree(sensor->name);
	}

	kfree(dev->sensors);
	dev->num_sensors = 0;
}

static __devinit int directemp_probe(struct usb_interface *interface,
			 const struct usb_device_id *id)
{
	struct usb_device *udev = interface_to_usbdev(interface);
	struct usb_directemp *dev;
	int err;

	dev = kzalloc(sizeof(struct usb_directemp), GFP_KERNEL);
	if (!dev)
		return -ENOMEM;

	dev->udev = usb_get_dev(udev);
	usb_set_intfdata(interface, dev);
	dev->usb_pid = id->idProduct;

	dev_info(&interface->dev, "Product:       %s (ID 0x%04X)\n",
			udev->product, dev->usb_pid);
	dev_info(&interface->dev, "Manufacturer:  %s\n",
			udev->manufacturer);
	dev_info(&interface->dev, "Serial Number: %s\n",
			udev->serial);

	mutex_init(&dev->lock);

	/* sysfs hooks */
	dev->hwmon_dev = hwmon_device_register(&interface->dev);
	if (IS_ERR(dev->hwmon_dev)) {
		dev_err(&interface->dev, "hwmon_device_register failed\n");
		err = PTR_ERR(dev->hwmon_dev);
		goto exit_free_usb;
	}

	err = device_create_file(&interface->dev, &dev_attr_raw);
	if (err)
		goto exit_free_sysfs;

	err = device_create_file(&interface->dev, &dev_attr_raw_simple);
	if (err)
		goto exit_free_sysfs;

	err = init_directemp_sensors(interface, dev);
	if (err)
		goto exit_free_sysfs;

	dev_info(&interface->dev, "DIRECTEMP(R) USB now attached\n");
	return 0;

exit_free_sysfs:
	hwmon_device_unregister(dev->hwmon_dev);
	dev_err(&interface->dev, "error while initializing driver\n");
	free_directemp_sensors(interface, dev);
	device_remove_file(&interface->dev, &dev_attr_hw_version);
	device_remove_file(&interface->dev, &dev_attr_sw_version);
	device_remove_file(&interface->dev, &dev_attr_serial_number);
	device_remove_file(&interface->dev, &dev_attr_raw_simple);
	device_remove_file(&interface->dev, &dev_attr_raw);
exit_free_usb:
	usb_set_intfdata(interface, NULL);
	usb_put_dev(dev->udev);
	kfree(dev);
	return err;
}

static __devexit void directemp_disconnect(struct usb_interface *interface)
{
	struct usb_directemp *dev = usb_get_intfdata(interface);

	free_directemp_sensors(interface, dev);
	device_remove_file(&interface->dev, &dev_attr_hw_version);
	device_remove_file(&interface->dev, &dev_attr_sw_version);
	device_remove_file(&interface->dev, &dev_attr_serial_number);
	device_remove_file(&interface->dev, &dev_attr_raw_simple);
	device_remove_file(&interface->dev, &dev_attr_raw);
	hwmon_device_unregister(dev->hwmon_dev);

	usb_set_intfdata(interface, NULL);
	usb_put_dev(dev->udev);

	kfree(dev);

	dev_info(&interface->dev, "disconnected\n");
}

static struct usb_device_id id_table[] = {
	{ USB_DEVICE(QTI_VENDOR_ID, 0x0002) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x0006) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x000B) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x000C) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x000D) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x000E) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x000F) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x0010) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x0011) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x0012) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x0013) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x0014) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x0015) },
	{ USB_DEVICE(QTI_VENDOR_ID, 0x0016) },
	{ }
};
MODULE_DEVICE_TABLE(usb, id_table);

static struct usb_driver directemp_driver = {
	.name		= DRV_NAME,
	.probe		= directemp_probe,
	.disconnect	= directemp_disconnect,
	.id_table	= id_table,
};

static int __init usb_directemp_init(void)
{
	static int version_printed = 0;
	int result;

	if (!version_printed) {
		pr_info(DRV_NAME ": DIRECTEMP(R) driver v" DRV_VERSION
				" (" DRV_RELDATE ")\n");
		version_printed = 1;
	}

	result = usb_register(&directemp_driver);
	if (result) {
		printk(KERN_ERR KBUILD_MODNAME ": usb_register failed! "
				"Error number: %d\n", result);
		return result;
	}

	return 0;
}

static void __exit usb_directemp_exit(void)
{
	usb_deregister(&directemp_driver);
}

module_init(usb_directemp_init);
module_exit(usb_directemp_exit);

MODULE_AUTHOR(DRV_AUTHOR);
MODULE_DESCRIPTION(DRV_DESCRIPTION);
MODULE_VERSION(DRV_VERSION);
MODULE_LICENSE("GPL");
